package il.ac.tau.cs.sw1.address.book;


/**
 * Defines an address book The address book contains contacts ordered
 * alphabetically (case insensitive)
 */
public interface IAddressBook extends Iterable<Contact> {

	/**
	 * Adds a contact to the address book. If a contact exists that has the same
	 * (case insensitive) name as the input an IllegalArgumentException should
	 * be thrown.
	 * 
	 * @param c
	 *            - the contact to be added
	 * 
	 * @pre c != null
	 * @pre c.getName() != null
	 * @pre !exists(c.getName())
	 */
	public void add(Contact c);

	/**
	 * Removes a contact from the address book. If a contact with the specified
	 * name (case insensitive) exists it will be removed, otherwise the method
	 * does nothing.
	 * 
	 * @param name
	 *            - the name of the contact to be removed
	 * 
	 * @pre name != null
	 */
	public void delete(String name);

	/**
	 * Returns {@code true} if the address book contains a contact with the
	 * specified name (case insensitive).
	 * 
	 * @param name
	 *            - the name to be searched for
	 * @return {@code true} if the address book contains a contact with the
	 *         specified name (case insensitive)
	 * 
	 * @pre name != null
	 */
	public boolean exists(String name);

	/**
	 * Returns the contact for the specified name.
	 * 
	 * @param name
	 *            - the name of the person
	 * @return the contact with this name, or {@code null} if no such contact
	 *         exists.
	 * 
	 * @pre name != null
	 */
	public Contact get(String name);

	/**
	 * Search for contacts whose name starts with the given prefix. The result
	 * is an iterable of these contacts. The order is by names, alphabetically
	 * (case insensitive)
	 * 
	 * @param prefix
	 *            - the name prefix to be searched for
	 * @return an iterable of contacts ordered alphabetically by name (case
	 *         insensitive)
	 */
	public Iterable<Contact> search(String prefix);

	/**
	 * Returns the number of contacts currently stored in the address book
	 * 
	 * @return the number of contacts currently stored in the address book
	 */
	public int size();
}
